window.Gleam.initSearch([{"doc":"polly","title":"polly","content":"# polly\n\n[![Package Version](https://img.shields.io/hexpm/v/polly)](https://hex.pm/packages/polly)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/polly/)\n\nPolly is a portable, polling file-system watcher, supporting all targets that [simplifile](https://hexdocs.pm/simplifile/simplifile.html) supports.\n\nWhile she may not be the fastest, she will help you in all situations where the [fs package](https://hex.pm/packages/fs) might not be suitable or supported. When needing to watch files over the network, or in situations where NIFs pose a security risk, cannot be compiled, or external dependencies like `inotify-tools` cannot be relied upon, Polly is here to help.\n\n```sh\ngleam add polly@2\n```\n\n```gleam\nimport gleam/io\nimport polly\n\npub fn main() {\n  polly.new()\n  |&gt; polly.add_dir(&quot;src&quot;)\n  |&gt; polly.interval(1000)\n  |&gt; polly.watch(fn(event) {\n    case event {\n      Changed(path) -&gt; io.println(&quot;CHANGED &quot; &lt;&gt; path)\n      Created(path) -&gt; io.println(&quot;CREATED &quot; &lt;&gt; path)\n      Deleted(path) -&gt; io.println(&quot;DELETED &quot; &lt;&gt; path)\n      Error(_, _) -&gt; Nil\n    }\n  })\n}\n```\n\n### Performance\n\nCompared to the [fs](https://hex.pm/packages/fs) package, Polly does not use specialised operating system APIs like [inotify](https://www.man7.org/linux/man-pages/man7/inotify.7.html) or [ReadDirectoryChangesW](https://learn.microsoft.com/en-us/windows/win32/api/winbase/nf-winbase-readdirectorychangesw). Instead, she recursively walks the specified directories every time the configured interval elapses, and compares the timestamps of all files with their previous timestamps that she needs to remember. Instead of having to keep around a single integer per directory, Polly stores all the file and directory names and their last timestamps, basically keeping around a &quot;virtual&quot; copy of the state of the filesystem.\n\nThis approach is fine if you&#39;re planning to watch single files or configuration/source directories, but even just forgetting to filter out the `build` directory can significantly slow down Polly and increase her CPU and memory usage a lot. **Never** attempt to use Polly to watch the entire file system, or even just the home directory of the user.\n\nAlways make sure you can&#39;t use other approaches first, and that you watch the smallest possible set of directories while having good [filter](./polly.html#filter) and [max_depth](./polly.html#max_depth) settings.\n\n","url":"index.html"},{"doc":"polly","title":"Event","content":"pub type Event {\n  Created(path: String)\n  Changed(path: String)\n  Deleted(path: String)\n  Error(path: String, reason: FileError)\n}\n A filesystem event.\n \n Polly is careful when handing you events, and makes sure that you always get\n them in the right order, so if a dictionary with some files was created,\n you will get the dictionary event first, before any children.\nCreated(path: String)\n A new file or dictionary was created!\n\nChanged(path: String)\n A file got modified!\n\nDeleted(path: String)\n A file or dictionary got deleted :(\n\nError(path: String, reason: FileError)\n An unexpected simplifile error happened :(\n \n Polly treats `Enoent` and `Eacces` errors as if the file got deleted,\n and will therefore never pass those to you.\n\nSynonyms:\npolly.Event\npolly Event","url":"polly.html#Event"},{"doc":"polly","title":"HasWatchedDirs","content":"pub type HasWatchedDirs\n A type that Polly uses to tag her options, indicating that you have\n called `add_dir` or `add_file`.\n\nSynonyms:\npolly.HasWatchedDirs\npolly HasWatchedDirs","url":"polly.html#HasWatchedDirs"},{"doc":"polly","title":"NoWatchedDirs","content":"pub type NoWatchedDirs\n A type that Polly uses to tag her options, indicating that you haven&#39;t\n called `add_dir` or `add_file` yet.\n\nSynonyms:\npolly.NoWatchedDirs\npolly NoWatchedDirs","url":"polly.html#NoWatchedDirs"},{"doc":"polly","title":"Options","content":"pub opaque type Options(has_watched_dirs)\n Polly uses the builder pattern to construct a watcher.\n \n The `has_watched_dirs` phantom type variable helps Polly keep track of\n wether or not you&#39;ve called `add_dir` or `add_file` yet. That way, it will\n be a type error if you try to watch nothing!\n \n \n let options: Options = polly.new()\n     |&gt; polly.add_dir(&quot;src&quot;)\n     |&gt; polly.interval(3000)\n     |&gt; polly.max_depth(10)\n     |&gt; polly.filter(polly.default_filter)\n \n\nSynonyms:\npolly.Options\npolly Options","url":"polly.html#Options"},{"doc":"polly","title":"Watcher","content":"pub opaque type Watcher\n A polling file system watcher, built from options.\n \n Watchers are automatically started, and can by stopped by calling `stop`.\n\nSynonyms:\npolly.Watcher\npolly Watcher","url":"polly.html#Watcher"},{"doc":"polly","title":"add_dir","content":"pub fn add_dir(\n  options: Options(a),\n  path: String,\n) -&gt; Options(HasWatchedDirs)\n Tell Polly which directory to watch. If it does not exist, `watch` will return an error.\n \n If the directory goes away after watching has started, Polly will continue to\n check on it to see if it came back.\n \n Paths are not expanded by default, so the paths reported by events and passed\n to the filter function will be prefixed with whatever you specified here.\nSynonyms:\npolly.add_dir\npolly add_dir","url":"polly.html#add_dir"},{"doc":"polly","title":"add_file","content":"pub fn add_file(\n  options: Options(a),\n  path: String,\n) -&gt; Options(HasWatchedDirs)\n Tell Polly to watch a single file.\n \n Polly doesn&#39;t care if you tell her to watch a file or directory, but\n using this function makes your intent clearer!\nSynonyms:\npolly.add_file\npolly add_file","url":"polly.html#add_file"},{"doc":"polly","title":"default_filter","content":"pub fn default_filter(type: FileType, path: String) -&gt; Bool\n The default filter function, ignoring hidden files starting with a colon `&quot;.&quot;`\nSynonyms:\npolly.default_filter\npolly default_filter","url":"polly.html#default_filter"},{"doc":"polly","title":"filter","content":"pub fn filter(\n  options: Options(a),\n  by filter: fn(FileType, String) -&gt; Bool,\n) -&gt; Options(a)\n Filter files using the given predicate.\n \n Polly will ignore files and directories for which the predicate returns `False`\n completely, and any event happening for them or for a contained file of them\n will not get reported.\n \n Keep in mind that the filter is checked for every part of a path, not just\n leaf nodes! So for example, if you have a path `./src/app.gleam`, your filter\n function will first be called on `.`, then on `./src`, and then finally on\n `./src/app.gleam`.\n \n By default, all hidden files are ignored by using the `default_filter`.\nSynonyms:\npolly.filter\npolly filter","url":"polly.html#filter"},{"doc":"polly","title":"ignore_initial_missing","content":"pub fn ignore_initial_missing(options: Options(a)) -&gt; Options(a)\n Tell Polly that it is fine if a file or directory does not exist initially.\n \n By default, if a file or directory cannot be found when calling `watch`,\n Polly will immediately return to you with an `Enoent` error and refuse to run.\n \n When this option is active, Polly will instead note the missing directory,\n and continuously check if it appears, similarly to how she does after a\n file or directory goes away after she has first seen it.\nSynonyms:\npolly.ignore_initial_missing\npolly ignore_initial_missing","url":"polly.html#ignore_initial_missing"},{"doc":"polly","title":"interval","content":"pub fn interval(options: Options(a), interval: Int) -&gt; Options(a)\n Set the interval in-between file-system polls, in milliseconds.\n \n This is the time that Polly rests between calls, so if scanning your directory\n tree takes 100ms, and you configured 1000ms here, the total time between calls\n will roughly be 1100ms.\n \n Doing it this way makes sure that Polly doesn&#39;t stumble over herself.\nSynonyms:\npolly.interval\npolly interval","url":"polly.html#interval"},{"doc":"polly","title":"max_depth","content":"pub fn max_depth(\n  options: Options(a),\n  max_depth: Int,\n) -&gt; Options(a)\n Limit the maximum depth that Polly will walk each directory.\n\n A limit of `0` would mean that Polly _only_ watches the specified list of\n files or directories. A limit of `1` means that she will also look at the\n files inside the given directories, but not at any nested directories.\n \n There is no limit by default, but setting a limit might be good to\n better control resource usage of the watcher.\n \n Calling this function multiple times will cause polly to only remember the\n lowest limit provided.\nSynonyms:\npolly.max_depth\npolly max_depth","url":"polly.html#max_depth"},{"doc":"polly","title":"new","content":"pub fn new() -&gt; Options(NoWatchedDirs)\n Start creating a new configuration using the default options.\n \n By default, an interval of 1 second is set, and the `default_filter` is used.\nSynonyms:\npolly.new\npolly new","url":"polly.html#new"},{"doc":"polly","title":"stop","content":"pub fn stop(watcher: Watcher) -&gt; Nil\n Stop this watcher.\n \n If Polly currently scans your directories, she might not hear you right away\n and may still report events for one run, after which she will stop.\nSynonyms:\npolly.stop\npolly stop","url":"polly.html#stop"},{"doc":"polly","title":"watch","content":"pub fn watch(\n  options: Options(HasWatchedDirs),\n  with emit: fn(Event) -&gt; a,\n) -&gt; Result(Watcher, List(#(String, FileError)))\n Tell Polly to start watching all the specified directories for changes.\n \n The callback is called synchronously while collecting change events since\n the last run. It is adviseable to move heavier cpu-bound tasks from this\n callback into their own processes or threads.\n \n When running on the Erlang target, this spawns a new process.\nSynonyms:\npolly.watch\npolly watch","url":"polly.html#watch"},{"doc":"polly","title":"watch_with","content":"pub fn watch_with(\n  options: Options(HasWatchedDirs),\n  from initial: a,\n  with emit: fn(a, Event) -&gt; a,\n) -&gt; Result(Watcher, List(#(String, FileError)))\n /// Like `watch`, but similar to `list.fold` Polly will also keep some state\n around for you and pass it back on each invocation.\nSynonyms:\npolly.watch_with\npolly watch_with","url":"polly.html#watch_with"},{"doc":"polly","title":"polly","content":"","url":"polly.html"}]);